@@ -12,7 +12,7 @@ use Encode;
 sub printheader ($) {
 	my $session=shift;
 	
-	if ($config{sslcookie}) {
+	if ($ENV{HTTPS} || $config{sslcookie}) {
 		print $session->header(-charset => 'utf-8',
 			-cookie => $session->cookie(-httponly => 1, -secure => 1));
 	}
@@ -46,13 +46,51 @@ sub showform ($$$$;@) {
 	my $cgi=shift;
 
 	printheader($session);
-	print misctemplate($form->title, $form->render(submit => $buttons), @_);
+	print cgitemplate($cgi, $form->title,
+		$form->render(submit => $buttons), @_);
+}
+
+sub cgitemplate ($$$;@) {
+	my $cgi=shift;
+	my $title=shift;
+	my $content=shift;
+	my %params=@_;
+	
+	my $template=template("page.tmpl");
+
+	my $topurl = defined $cgi ? $cgi->url : $config{url};
+
+	my $page="";
+	if (exists $params{page}) {
+		$page=delete $params{page};
+		$params{forcebaseurl}=urlabs(urlto($page), $topurl);
+	}
+	run_hooks(pagetemplate => sub {
+		shift->(
+			page => $page,
+			destpage => $page,
+			template => $template,
+		);
+	});
+	templateactions($template, "");
+
+	$template->param(
+		dynamic => 1,
+		title => $title,
+		wikiname => $config{wikiname},
+		content => $content,
+		baseurl => urlabs(baseurl(), $topurl),
+		html5 => $config{html5},
+		%params,
+	);
+	
+	return $template->output;
 }
 
 sub redirect ($$) {
 	my $q=shift;
 	eval q{use URI};
-	my $url=URI->new(shift);
+	my $url=URI->new(urlabs(shift, $q->url));
 	if (! $config{w3mmode}) {
 		print $q->redirect($url);
 	}
@@ -116,7 +154,7 @@ sub cgi_signin ($$;$) {
 		required => 'NONE',
 		javascript => 0,
 		params => $q,
-		action => $config{cgiurl},
+		action => cgiurl(),
 		header => 0,
 		template => {type => 'div'},
 		stylesheet => 1,
@@ -198,7 +236,7 @@ sub cgi_prefs ($$) {
 		required => 'NONE',
 		javascript => 0,
 		params => $q,
-		action => $config{cgiurl},
+		action => cgiurl(),
 		template => {type => 'div'},
 		stylesheet => 1,
 		fieldsets => [
@@ -231,11 +269,11 @@ sub cgi_prefs ($$) {
 	
 	if ($form->submitted eq 'Logout') {
 		$session->delete();
-		redirect($q, $config{url});
+		redirect($q, baseurl(undef));
 		return;
 	}
 	elsif ($form->submitted eq 'Cancel') {
-		redirect($q, $config{url});
+		redirect($q, baseurl(undef));
 		return;
 	}
 	elsif ($form->submitted eq 'Save Preferences' && $form->validate) {
@@ -283,7 +321,7 @@ sub check_banned ($$) {
 
 	foreach my $b (@{$config{banned_users}}) {
 		if (pagespec_match("", $b,
-			ip => $ENV{REMOTE_ADDR},
+			ip => $session->remote_addr(),
 			name => defined $name ? $name : "",
 		)) {
 			$banned=1;
@@ -423,7 +461,7 @@ sub cgierror ($) {
 	my $message=shift;
 
 	print "Content-type: text/html\n\n";
-	print misctemplate(gettext("Error"),
+	print cgitemplate(undef, gettext("Error"),
 		"<p class=\"error\">".gettext("Error").": $message</p>");
 	die $@;
 }
