@@ -58,12 +58,21 @@ sub getsetup () {
 			safe => 1,
 			rebuild => 0,
 		},
+		cookiejar => {
+			type => "string",
+			example => { file => "$ENV{HOME}/.ikiwiki/cookies" },
+			safe => 0, # hooks into perl module internals
+			description => "cookie control",
+		},
 }
 
 sub checkconfig () {
 	if (! defined $config{aggregateinternal}) {
 		$config{aggregateinternal}=1;
 	}
+	if (! defined $config{cookiejar}) {
+		$config{cookiejar}={ file => "$ENV{HOME}/.ikiwiki/cookies" };
+	}
 
 	# This is done here rather than in a refresh hook because it
 	# needs to run before the wiki is locked.
@@ -104,7 +113,8 @@ sub launchaggregation () {
 	my @feeds=needsaggregate();
 	return unless @feeds;
 	if (! lockaggregate()) {
-		error("an aggregation process is already running");
+		debug("an aggregation process is already running");
+		return;
 	}
 	# force a later rebuild of source pages
 	$IkiWiki::forcerebuild{$_->{sourcepage}}=1
@@ -191,7 +201,7 @@ sub migrate_to_internal {
 		if (-e $oldoutput) {
 			require IkiWiki::Render;
 			debug("removing output file $oldoutput");
-			IkiWiki::prune($oldoutput, $config{destdir});
+			IkiWiki::prune($oldoutput);
 		}
 	}
 	
@@ -488,7 +498,6 @@ sub needsaggregate () {
 }
 
 sub aggregate (@) {
-	eval q{use Net::INET6Glue::INET_is_INET6}; # may not be available
 	eval q{use XML::Feed};
 	error($@) if $@;
 	eval q{use URI::Fetch};
@@ -513,8 +522,11 @@ sub aggregate (@) {
 			}
 			$feed->{feedurl}=pop @urls;
 		}
-		my $ua=useragent();
-		my $res=URI::Fetch->fetch($feed->{feedurl}, UserAgent=>$ua);
+		my $res=URI::Fetch->fetch($feed->{feedurl},
+			UserAgent => LWP::UserAgent->new(
+				cookie_jar => $config{cookiejar},
+			),
+		);
 		if (! $res) {
 			$feed->{message}=URI::Fetch->errstr;
 			$feed->{error}=1;
@@ -553,9 +565,7 @@ sub aggregate (@) {
 			};
 		}
 		if ($@) {
-			# gettext can clobber $@
-			my $error = $@;
-			$feed->{message}=gettext("feed crashed XML::Feed!")." ($error)";
+			$feed->{message}=gettext("feed crashed XML::Feed!")." ($@)";
 			$feed->{error}=1;
 			debug($feed->{message});
 			next;
@@ -583,7 +593,6 @@ sub aggregate (@) {
 				feed => $feed,
 				copyright => $f->copyright,
 				title => defined $entry->title ? decode_entities($entry->title) : "untitled",
-				author => defined $entry->author ? decode_entities($entry->author) : "",
 				link => $entry->link,
 				content => (defined $c && defined $c->body) ? $c->body : "",
 				guid => defined $entry->id ? $entry->id : time."_".$feed->{name},
@@ -604,7 +613,6 @@ sub add_page (@) {
 		# updating an existing post
 		$guid=$guids{$params{guid}};
 		return if $guid->{expired};
-		write_page($feed, $guid, $mtime, \%params);
 	}
 	else {
 		# new post
@@ -630,35 +638,24 @@ sub add_page (@) {
 		       -e "$config{srcdir}/".htmlfn($page.$c)) {
 			$c++
 		}
-		$page=$page.$c;
 
-		$guid->{page}=$page;
-		eval { write_page($feed, $guid, $mtime, \%params) };
-		if ($@) {
-			# assume failure was due to a too long filename
+		# Make sure that the file name isn't too long. 
+		# NB: This doesn't check for path length limits.
+		my $max=POSIX::pathconf($config{srcdir}, &POSIX::_PC_NAME_MAX);
+		if (defined $max && length(htmlfn($page)) >= $max) {
 			$c="";
 			$page=$feed->{dir}."/item";
 			while (exists $IkiWiki::pagecase{lc $page.$c} ||
 			      -e $IkiWiki::Plugin::transient::transientdir."/".htmlfn($page.$c) ||
-			      -e "$config{srcdir}/".htmlfn($page.$c)) {
+
+			       -e "$config{srcdir}/".htmlfn($page.$c)) {
 				$c++
 			}
-			$page=$page.$c;
-
-			$guid->{page}=$page;
-			write_page($feed, $guid, $mtime, \%params);
 		}
 
+		$guid->{page}=$page;
 		debug(sprintf(gettext("creating new page %s"), $page));
 	}
-}
-
-sub write_page ($$$$$) {
-	my $feed=shift;
-	my $guid=shift;
-	my $mtime=shift;
-	my %params=%{shift()};
-
 	$guid->{feed}=$feed->{name};
 	
 	# To write or not to write? Need to avoid writing unchanged pages
@@ -677,16 +674,11 @@ sub write_page ($$$$$) {
 		$template=template($feed->{template}, blind_cache => 1);
 	};
 	if ($@) {
-		# gettext can clobber $@
-		my $error = $@;
-		print STDERR gettext("failed to process template:")." $error";
+		print STDERR gettext("failed to process template:")." $@";
 		return;
 	}
 	$template->param(title => $params{title})
 		if defined $params{title} && length($params{title});
-	$template->param(author => $params{author})
-		if defined $params{author} && length($params{author}
-			&& $params{author} ne $feed->{name});
 	$template->param(content => wikiescape(htmlabs($params{content},
 		defined $params{base} ? $params{base} : $feed->{feedurl})));
 	$template->param(name => $feed->{name});
