@@ -75,7 +75,7 @@ sub cgi_editpage ($$) {
 		required => [qw{editcontent}],
 		javascript => 0,
 		params => $q,
-		action => $config{cgiurl},
+		action => IkiWiki::cgiurl(),
 		header => 0,
 		table => 0,
 		template => { template("editpage.tmpl") },
@@ -91,6 +91,9 @@ sub cgi_editpage ($$) {
 	# This untaint is safe because we check file_pruned and
 	# wiki_file_regexp.
 	my ($page)=$form->field('page')=~/$config{wiki_file_regexp}/;
+	if (! defined $page) {
+		error(gettext("bad page name"));
+	}
 	$page=possibly_foolish_untaint($page);
 	my $absolute=($page =~ s#^/+##); # absolute name used to force location
 	if (! defined $page || ! length $page ||
@@ -98,7 +101,7 @@ sub cgi_editpage ($$) {
 		error(gettext("bad page name"));
 	}
 
-	my $baseurl = urlto($page, undef, 1);
+	my $baseurl = urlto($page);
 
 	my $from;
 	if (defined $form->field('from')) {
@@ -128,7 +131,8 @@ sub cgi_editpage ($$) {
 			# favor the type of linking page
 			$type=pagetype($pagesources{$from});
 		}
-		$type=$config{default_pageext} unless defined $type;
+		$type=$config{default_pageext}
+			if ! defined $type || $type=~/^_/; # not internal type
 		$file=newpagefile($page, $type);
 		if (! $form->submitted) {
 			$form->field(name => "rcsinfo", value => "", force => 1);
@@ -156,13 +160,13 @@ sub cgi_editpage ($$) {
 	my $previewing=0;
 	if ($form->submitted eq "Cancel") {
 		if ($form->field("do") eq "create" && defined $from) {
-			redirect($q, urlto($from, undef, 1));
+			redirect($q, urlto($from));
 		}
 		elsif ($form->field("do") eq "create") {
-			redirect($q, $config{url});
+			redirect($q, baseurl(undef));
 		}
 		else {
-			redirect($q, urlto($page, undef, 1));
+			redirect($q, $baseurl);
 		}
 		exit;
 	}
@@ -262,7 +266,7 @@ sub cgi_editpage ($$) {
 					@page_locs=$page;
 				}
 				else {
-					redirect($q, urlto($page, undef, 1));
+					redirect($q, $baseurl);
 					exit;
 				}
 			}
@@ -291,7 +295,7 @@ sub cgi_editpage ($$) {
 				value => $best_loc);
 			$form->field(name => "type", type => 'select',
 				options => \@page_types);
-			$form->title(sprintf(gettext("creating %s"), pagetitle($page)));
+			$form->title(sprintf(gettext("creating %s"), pagetitle(basename($page))));
 			
 		}
 		elsif ($form->field("do") eq "edit") {
@@ -309,11 +313,10 @@ sub cgi_editpage ($$) {
 			$form->tmpl_param("page_select", 0);
 			$form->field(name => "page", type => 'hidden');
 			$form->field(name => "type", type => 'hidden');
-			$form->title(sprintf(gettext("editing %s"), pagetitle($page)));
+			$form->title(sprintf(gettext("editing %s"), pagetitle(basename($page))));
 		}
 		
-		showform($form, \@buttons, $session, $q,
-			forcebaseurl => $baseurl, page => $page);
+		showform($form, \@buttons, $session, $q, page => $page);
 	}
 	else {
 		# save page
@@ -331,7 +334,7 @@ sub cgi_editpage ($$) {
 			$form->field(name => "type", type => 'hidden');
 			$form->title(sprintf(gettext("editing %s"), $page));
 			showform($form, \@buttons, $session, $q,
-				forcebaseurl => $baseurl, page => $page);
+				page => $page);
 			exit;
 		}
 		elsif ($form->field("do") eq "create" && $exists) {
@@ -346,7 +349,7 @@ sub cgi_editpage ($$) {
 				         "\n\n\n".$form->field("editcontent"),
 				force => 1);
 			showform($form, \@buttons, $session, $q,
-				forcebaseurl => $baseurl, page => $page);
+				page => $page);
 			exit;
 		}
 			
@@ -387,7 +390,7 @@ sub cgi_editpage ($$) {
 			$form->field(name => "type", type => 'hidden');
 			$form->title(sprintf(gettext("editing %s"), $page));
 			showform($form, \@buttons, $session, $q,
-				forcebaseurl => $baseurl, page => $page);
+				page => $page);
 			exit;
 		}
 		
@@ -401,9 +404,12 @@ sub cgi_editpage ($$) {
 			# signaling to it that it should not try to
 			# do anything.
 			disable_commit_hook();
-			$conflict=rcs_commit($file, $message,
-				$form->field("rcsinfo"),
-				$session->param("name"), $ENV{REMOTE_ADDR});
+			$conflict=rcs_commit(
+				file => $file,
+				message => $message,
+				token => $form->field("rcsinfo"),
+				session => $session,
+			);
 			enable_commit_hook();
 			rcs_update();
 		}
@@ -426,12 +432,12 @@ sub cgi_editpage ($$) {
 			$form->field(name => "type", type => 'hidden');
 			$form->title(sprintf(gettext("editing %s"), $page));
 			showform($form, \@buttons, $session, $q,
-				forcebaseurl => $baseurl, page => $page);
+				page => $page);
 		}
 		else {
 			# The trailing question mark tries to avoid broken
 			# caches and get the most recent version of the page.
-			redirect($q, urlto($page, undef, 1)."?updated");
+			redirect($q, $baseurl."?updated");
 		}
 	}
 
