@@ -104,7 +104,7 @@ sub checkconfig () {
 }
 
 sub format (@) {
-        my %params=@_;
+	my %params=@_;
 
 	# Fill in the inline content generated earlier. This is actually an
 	# optimisation.
@@ -128,10 +128,10 @@ sub sessioncgi ($$) {
 			$add=1 unless length $add;
 			$add++;
 		}
-		$q->param('page', $page.$add);
+		$q->param('page', "/$from/$page$add");
 		# now go create the page
 		$q->param('do', 'create');
-		# make sure the editpage plugin in loaded
+		# make sure the editpage plugin is loaded
 		if (IkiWiki->can("cgi_editpage")) {
 			IkiWiki::cgi_editpage($q, $session);
 		}
@@ -160,7 +160,7 @@ sub preprocess_inline (@) {
 	my $rss=(($config{rss} || $config{allowrss}) && exists $params{rss}) ? yesno($params{rss}) : $config{rss};
 	my $atom=(($config{atom} || $config{allowatom}) && exists $params{atom}) ? yesno($params{atom}) : $config{atom};
 	my $quick=exists $params{quick} ? yesno($params{quick}) : 0;
-	my $feeds=! $nested && (exists $params{feeds} ? yesno($params{feeds}) : !$quick);
+	my $feeds=exists $params{feeds} ? yesno($params{feeds}) : !$quick && ! $raw;
 	my $emptyfeeds=exists $params{emptyfeeds} ? yesno($params{emptyfeeds}) : 1;
 	my $feedonly=yesno($params{feedonly});
 	if (! exists $params{show} && ! $archive) {
@@ -269,7 +269,7 @@ sub preprocess_inline (@) {
 			}
 			$params{feedfile}=possibly_foolish_untaint($params{feedfile});
 		}
-		$feedbase=targetpage($params{destpage}, "", $params{feedfile});
+		$feedbase=targetpage($params{page}, "", $params{feedfile});
 
 		my $feedid=join("\0", $feedbase, map { $_."\0".$params{$_} } sort keys %params);
 		if (exists $knownfeeds{$feedid}) {
@@ -290,8 +290,17 @@ sub preprocess_inline (@) {
 		}
 	}
 
-	my $rssurl=abs2rel($feedbase."rss".$feednum, dirname(htmlpage($params{destpage}))) if $feeds && $rss;
-	my $atomurl=abs2rel($feedbase."atom".$feednum, dirname(htmlpage($params{destpage}))) if $feeds && $atom;
+	my ($rssurl, $atomurl, $rssdesc, $atomdesc);
+	if ($feeds) {
+		if ($rss) {
+			$rssurl=abs2rel($feedbase."rss".$feednum, dirname(htmlpage($params{destpage})));
+			$rssdesc = sprintf(gettext("%s (RSS feed)"), $desc);
+		}
+		if ($atom) {
+			$atomurl=abs2rel($feedbase."atom".$feednum, dirname(htmlpage($params{destpage})));
+			$atomdesc = sprintf(gettext("%s (Atom feed)"), $desc);
+		}
+	}
 
 	my $ret="";
 
@@ -299,11 +308,19 @@ sub preprocess_inline (@) {
 	    (exists $params{postform} && yesno($params{postform}))) &&
 	    IkiWiki->can("cgi_editpage")) {
 		# Add a blog post form, with feed buttons.
-		my $formtemplate=template("blogpost.tmpl", blind_cache => 1);
-		$formtemplate->param(cgiurl => $config{cgiurl});
+		my $formtemplate=template_depends("blogpost.tmpl", $params{page}, blind_cache => 1);
+		$formtemplate->param(cgiurl => IkiWiki::cgiurl());
 		$formtemplate->param(rootpage => rootpage(%params));
-		$formtemplate->param(rssurl => $rssurl) if $feeds && $rss;
-		$formtemplate->param(atomurl => $atomurl) if $feeds && $atom;
+		if ($feeds) {
+			if ($rss) {
+				$formtemplate->param(rssurl => $rssurl);
+				$formtemplate->param(rssdesc => $rssdesc);
+			}
+			if ($atom) {
+				$formtemplate->param(atomurl => $atomurl);
+				$formtemplate->param(atomdesc => $atomdesc);
+			}
+		}
 		if (exists $params{postformtext}) {
 			$formtemplate->param(postformtext =>
 				$params{postformtext});
@@ -312,6 +329,10 @@ sub preprocess_inline (@) {
 			$formtemplate->param(postformtext =>
 				gettext("Add a new post titled:"));
 		}
+		if (exists $params{id}) {
+			$formtemplate->param(postformid =>
+				$params{id});
+		}
 		$ret.=$formtemplate->output;
 	    	
 		# The post form includes the feed buttons, so
@@ -320,19 +341,34 @@ sub preprocess_inline (@) {
 	}
 	elsif ($feeds && !$params{preview} && ($emptyfeeds || @feedlist)) {
 		# Add feed buttons.
-		my $linktemplate=template("feedlink.tmpl", blind_cache => 1);
-		$linktemplate->param(rssurl => $rssurl) if $rss;
-		$linktemplate->param(atomurl => $atomurl) if $atom;
+		my $linktemplate=template_depends("feedlink.tmpl", $params{page}, blind_cache => 1);
+		if ($rss) {
+			$linktemplate->param(rssurl => $rssurl);
+			$linktemplate->param(rssdesc => $rssdesc);
+		}
+		if ($atom) {
+			$linktemplate->param(atomurl => $atomurl);
+			$linktemplate->param(atomdesc => $atomdesc);
+		}
+		if (exists $params{id}) {
+			$linktemplate->param(id => $params{id});
+		}
 		$ret.=$linktemplate->output;
 	}
 	
 	if (! $feedonly) {
-		require HTML::Template;
-		my @params=IkiWiki::template_params($params{template}.".tmpl", blind_cache => 1);
-		if (! @params) {
-			error sprintf(gettext("nonexistant template %s"), $params{template});
+		my $template;
+		if (! $raw) {
+			# cannot use wiki pages as templates; template not sanitized due to
+			# format hook hack
+			eval {
+				$template=template_depends($params{template}.".tmpl", $params{page},
+					blind_cache => 1);
+			};
+			if ($@) {
+				error sprintf(gettext("failed to process template %s"), $params{template}.".tmpl").": $@";
+			}
 		}
-		my $template=HTML::Template->new(@params) unless $raw;
 		my $needcontent=$raw || (!($archive && $quick) && $template->query(name => 'content'));
 	
 		foreach my $page (@list) {
@@ -349,10 +385,11 @@ sub preprocess_inline (@) {
 				$template->param(pageurl => urlto($page, $params{destpage}));
 				$template->param(inlinepage => $page);
 				$template->param(title => pagetitle(basename($page)));
-				$template->param(ctime => displaytime($pagectime{$page}, $params{timeformat}));
+				$template->param(ctime => displaytime($pagectime{$page}, $params{timeformat}, 1));
 				$template->param(mtime => displaytime($pagemtime{$page}, $params{timeformat}));
 				$template->param(first => 1) if $page eq $list[0];
 				$template->param(last => 1) if $page eq $list[$#list];
+				$template->param(html5 => $config{html5});
 	
 				if ($actions) {
 					my $file = $pagesources{$page};
@@ -410,9 +447,9 @@ sub preprocess_inline (@) {
 			if (! $params{preview}) {
 				writefile($rssp, $config{destdir},
 					genfeed("rss",
-						$config{url}."/".$rssp, $desc, $params{guid}, $params{destpage}, @feedlist));
+						$config{url}."/".$rssp, $desc, $params{guid}, $params{page}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
-				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/rss+xml" title="$desc (RSS)" href="$rssurl" />};
+				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/rss+xml" title="$rssdesc" href="$rssurl" />};
 			}
 		}
 		if ($atom) {
@@ -420,9 +457,9 @@ sub preprocess_inline (@) {
 			will_render($params{destpage}, $atomp);
 			if (! $params{preview}) {
 				writefile($atomp, $config{destdir},
-					genfeed("atom", $config{url}."/".$atomp, $desc, $params{guid}, $params{destpage}, @feedlist));
+					genfeed("atom", $config{url}."/".$atomp, $desc, $params{guid}, $params{page}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
-				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/atom+xml" title="$desc (Atom)" href="$atomurl" />};
+				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/atom+xml" title="$atomdesc" href="$atomurl" />};
 			}
 		}
 	}
@@ -466,6 +503,13 @@ sub get_inline_content ($$) {
 		       filter($page, $destpage,
 		       readfile(srcfile($file))))));
 		$nested--;
+		if (isinternal($page)) {
+			# make inlined text of internal pages searchable
+			run_hooks(indexhtml => sub {
+				shift->(page => $page, destpage => $page,
+					content => $ret);
+			});
+		}
 	}
 	
 	if ($cached_destpage ne $destpage) {
@@ -491,37 +535,60 @@ sub date_822 ($) {
 	return $ret;
 }
 
-sub date_3339 ($) {
-	my $time=shift;
-
-	my $lc_time=POSIX::setlocale(&POSIX::LC_TIME);
-	POSIX::setlocale(&POSIX::LC_TIME, "C");
-	my $ret=POSIX::strftime("%Y-%m-%dT%H:%M:%SZ", gmtime($time));
-	POSIX::setlocale(&POSIX::LC_TIME, $lc_time);
-	return $ret;
-}
-
 sub absolute_urls ($$) {
-	# sucky sub because rss sucks
-	my $content=shift;
+	# needed because rss sucks
+	my $html=shift;
 	my $baseurl=shift;
 
 	my $url=$baseurl;
 	$url=~s/[^\/]+$//;
+	my $urltop; # calculated if needed
 
-        # what is the non path part of the url?
-        my $top_uri = URI->new($url);
-        $top_uri->path_query(""); # reset the path
-        my $urltop = $top_uri->as_string;
+	my $ret="";
 
-	$content=~s/(<a(?:\s+(?:class|id)\s*="?\w+"?)?)\s+href=\s*"(#[^"]+)"/$1 href="$baseurl$2"/mig;
-        # relative to another wiki page
-	$content=~s/(<a(?:\s+(?:class|id)\s*="?\w+"?)?)\s+href=\s*"(?!\w+:)([^\/][^"]*)"/$1 href="$url$2"/mig;
-	$content=~s/(<img(?:\s+(?:class|id|width|height)\s*="?\w+"?)*)\s+src=\s*"(?!\w+:)([^\/][^"]*)"/$1 src="$url$2"/mig;
-        # relative to the top of the site
-	$content=~s/(<a(?:\s+(?:class|id)\s*="?\w+"?)?)\s+href=\s*"(?!\w+:)(\/[^"]*)"/$1 href="$urltop$2"/mig;
-	$content=~s/(<img(?:\s+(?:class|id|width|height)\s*="?\w+"?)*)\s+src=\s*"(?!\w+:)(\/[^"]*)"/$1 src="$urltop$2"/mig;
-	return $content;
+	eval q{use HTML::Parser; use HTML::Tagset};
+	die $@ if $@;
+	my $p = HTML::Parser->new(api_version => 3);
+	$p->handler(default => sub { $ret.=join("", @_) }, "text");
+	$p->handler(start => sub {
+		my ($tagname, $pos, $text) = @_;
+		if (ref $HTML::Tagset::linkElements{$tagname}) {
+			while (4 <= @$pos) {
+				# use attribute sets from right to left
+				# to avoid invalidating the offsets
+				# when replacing the values
+				my ($k_offset, $k_len, $v_offset, $v_len) =
+					splice(@$pos, -4);
+				my $attrname = lc(substr($text, $k_offset, $k_len));
+				next unless grep { $_ eq $attrname } @{$HTML::Tagset::linkElements{$tagname}};
+				next unless $v_offset; # 0 v_offset means no value
+				my $v = substr($text, $v_offset, $v_len);
+				$v =~ s/^([\'\"])(.*)\1$/$2/;
+				if ($v=~/^#/) {
+					$v=$baseurl.$v; # anchor
+				}
+				elsif ($v=~/^(?!\w+:)[^\/]/) {
+					$v=$url.$v; # relative url
+				}
+				elsif ($v=~/^\//) {
+					if (! defined $urltop) {
+						# what is the non path part of the url?
+						my $top_uri = URI->new($url);
+						$top_uri->path_query(""); # reset the path
+						$urltop = $top_uri->as_string;
+					}
+					$v=$urltop.$v; # url relative to top of site
+				}
+				$v =~ s/\"/&quot;/g; # since we quote with ""
+				substr($text, $v_offset, $v_len) = qq("$v");
+			}
+		}
+		$ret.=$text;
+	}, "tagname, tokenpos, text");
+	$p->parse($html);
+	$p->eof;
+
+	return $ret;
 }
 
 sub genfeed ($$$$$@) {
@@ -534,7 +601,7 @@ sub genfeed ($$$$$@) {
 	
 	my $url=URI->new(encode_utf8(urlto($page,"",1)));
 	
-	my $itemtemplate=template($feedtype."item.tmpl", blind_cache => 1);
+	my $itemtemplate=template_depends($feedtype."item.tmpl", $page, blind_cache => 1);
 	my $content="";
 	my $lasttime = 0;
 	foreach my $p (@pages) {
@@ -553,7 +620,8 @@ sub genfeed ($$$$$@) {
 
 		if (exists $pagestate{$p}) {
 			if (exists $pagestate{$p}{meta}{guid}) {
-				$itemtemplate->param(guid => $pagestate{$p}{meta}{guid});
+				eval q{use HTML::Entities};
+				$itemtemplate->param(guid => HTML::Entities::encode_numeric($pagestate{$p}{meta}{guid}));
 			}
 
 			if (exists $pagestate{$p}{meta}{updated}) {
@@ -597,7 +665,7 @@ sub genfeed ($$$$$@) {
 		$lasttime = $pagemtime{$p} if $pagemtime{$p} > $lasttime;
 	}
 
-	my $template=template($feedtype."page.tmpl", blind_cache => 1);
+	my $template=template_depends($feedtype."page.tmpl", $page, blind_cache => 1);
 	$template->param(
 		title => $page ne "index" ? pagetitle($page) : $config{wikiname},
 		wikiname => $config{wikiname},
