@@ -37,12 +37,13 @@ sub needsbuild (@) {
 			}
 		}
 	}
+	return $needsbuild;
 }
 
-sub scrub ($$) {
+sub scrub ($$$) {
 	if (IkiWiki::Plugin::htmlscrubber->can("sanitize")) {
 		return IkiWiki::Plugin::htmlscrubber::sanitize(
-			content => shift, destpage => shift);
+			content => shift, page => shift, destpage => shift);
 	}
 	else {
 		return shift;
@@ -161,7 +162,7 @@ sub preprocess (@) {
 	# Metadata handling that happens only during preprocessing pass.
 	if ($key eq 'permalink') {
 		if (safeurl($value)) {
-			push @{$metaheaders{$page}}, scrub('<link rel="bookmark" href="'.encode_entities($value).'" />', $destpage);
+			push @{$metaheaders{$page}}, scrub('<link rel="bookmark" href="'.encode_entities($value).'" />', $page, $destpage);
 		}
 	}
 	elsif ($key eq 'stylesheet') {
@@ -173,10 +174,10 @@ sub preprocess (@) {
 		if (! length $stylesheet) {
 			error gettext("stylesheet not found")
 		}
-		push @{$metaheaders{$page}}, '<link href="'.urlto($stylesheet, $page).
+		push @{$metaheaders{$page}}, scrub('<link href="'.urlto($stylesheet, $page).
 			'" rel="'.encode_entities($rel).
 			'" title="'.encode_entities($title).
-			"\" type=\"text/css\" />";
+			"\" type=\"text/css\" />", $page, $destpage);
 	}
 	elsif ($key eq 'openid') {
 		my $delegate=0; # both by default
@@ -197,8 +198,12 @@ sub preprocess (@) {
 				'" rel="openid2.local_id" />' if $delegate ne 1;
 		}
 		if (exists $params{"xrds-location"} && safeurl($params{"xrds-location"})) {
-			push @{$metaheaders{$page}}, '<meta http-equiv="X-XRDS-Location"'.
-				'content="'.encode_entities($params{"xrds-location"}).'" />';
+			# force url absolute
+			eval q{use URI};
+			error($@) if $@;
+			my $url=URI->new_abs($params{"xrds-location"}, $config{url});
+			push @{$metaheaders{$page}}, '<meta http-equiv="X-XRDS-Location" '.
+				'content="'.encode_entities($url).'" />';
 		}
 	}
 	elsif ($key eq 'redir') {
@@ -235,7 +240,7 @@ sub preprocess (@) {
 		my $delay=int(exists $params{delay} ? $params{delay} : 0);
 		my $redir="<meta http-equiv=\"refresh\" content=\"$delay; URL=$value\" />";
 		if (! $safe) {
-			$redir=scrub($redir, $destpage);
+			$redir=scrub($redir, $page, $destpage);
 		}
 		push @{$metaheaders{$page}}, $redir;
 	}
@@ -245,7 +250,7 @@ sub preprocess (@) {
 				join(" ", map {
 					encode_entities($_)."=\"".encode_entities(decode_entities($params{$_}))."\""
 				} keys %params).
-				" />\n", $destpage);
+				" />\n", $page, $destpage);
 		}
 	}
 	elsif ($key eq 'robots') {
@@ -253,12 +258,20 @@ sub preprocess (@) {
 			' content="'.encode_entities($value).'" />';
 	}
 	elsif ($key eq 'description') {
-		push @{$metaheaders{$page}}, '<meta name="'.encode_entities($key).
+		push @{$metaheaders{$page}}, '<meta name="'.
+			encode_entities($key).
 			'" content="'.encode_entities($value).'" />';
 	}
+	elsif ($key eq 'name') {
+		push @{$metaheaders{$page}}, scrub('<meta '.$key.'="'.
+			encode_entities($value).
+			join(' ', map { "$_=\"$params{$_}\"" } keys %params).
+			' />', $page, $destpage);
+	}
 	else {
-		push @{$metaheaders{$page}}, scrub('<meta name="'.encode_entities($key).
-			'" content="'.encode_entities($value).'" />', $destpage);
+		push @{$metaheaders{$page}}, scrub('<meta name="'.
+			encode_entities($key).'" content="'.
+			encode_entities($value).'" />', $page, $destpage);
 	}
 
 	return "";
@@ -285,6 +298,11 @@ sub pagetemplate (@) {
 			if exists $pagestate{$page}{meta}{$field} && $template->query(name => $field);
 	}
 
+	foreach my $field (qw{permalink}) {
+		$template->param($field => IkiWiki::urlabs($pagestate{$page}{meta}{$field}, $config{url}))
+			if exists $pagestate{$page}{meta}{$field} && $template->query(name => $field);
+	}
+
 	foreach my $field (qw{description}) {
 		$template->param($field => HTML::Entities::encode_numeric($pagestate{$page}{meta}{$field}))
 			if exists $pagestate{$page}{meta}{$field} && $template->query(name => $field);
@@ -342,7 +360,7 @@ sub match {
 	}
 
 	if (defined $val) {
-		if ($val=~/^$re$/i) {
+		if ($val=~$re) {
 			return IkiWiki::SuccessReason->new("$re matches $field of $page", $page => $IkiWiki::DEPEND_CONTENT, "" => 1);
 		}
 		else {
