@@ -7,7 +7,8 @@ use IkiWiki 3.00;
 
 sub import {
 	hook(type => "getsetup", id => "opendiscussion", call => \&getsetup);
-	hook(type => "canedit", id => "opendiscussion", call => \&canedit);
+	hook(type => "canedit", id => "opendiscussion", call => \&canedit,
+		first => 1);
 }
 
 sub getsetup () {
@@ -15,6 +16,7 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => 0,
+			section => "auth",
 		},
 }
 
@@ -23,7 +25,8 @@ sub canedit ($$) {
 	my $cgi=shift;
 	my $session=shift;
 
-	return "" if $page=~/(\/|^)\Q$config{discussionpage}\E$/;
+	return "" if $page=~/(\/|^)\Q$config{discussionpage}\E$/i;
+	return "" if pagespec_match($page, "postcomment(*)");
 	return undef;
 }
 
