@@ -5,7 +5,6 @@ package IkiWiki;
 use warnings;
 use strict;
 use IkiWiki;
-use Encode;
 
 my (%backlinks, %rendered);
 our %brokenlinks;
@@ -94,7 +93,8 @@ sub genpage ($$) {
 	}
 	if (defined $config{historyurl} && length $config{historyurl}) {
 		my $u=$config{historyurl};
-		$u=~s/\[\[file\]\]/$pagesources{$page}/g;
+		my $p=uri_escape_utf8($pagesources{$page}, '^A-Za-z0-9\-\._~/');
+		$u=~s/\[\[file\]\]/$p/g;
 		$template->param(historyurl => $u);
 		$actions++;
 	}
@@ -174,15 +174,15 @@ sub scan ($) {
 		}
 		delete $typedlinks{$page};
 
+		# Preprocess in scan-only mode.
+		preprocess($page, $page, $content, 1);
+
 		run_hooks(scan => sub {
 			shift->(
 				page => $page,
 				content => $content,
 			);
 		});
-
-		# Preprocess in scan-only mode.
-		preprocess($page, $page, $content, 1);
 	}
 	else {
 		will_render($file, $file, 1);
@@ -374,7 +374,7 @@ sub find_new_files ($) {
 			}
 
 			eval {
-				my $ctime=rcs_getctime("$config{srcdir}/$file");
+				my $ctime=rcs_getctime($file);
 				if ($ctime > 0) {
 					$pagectime{$page}=$ctime;
 				}
@@ -384,7 +384,7 @@ sub find_new_files ($) {
 			}
 			my $mtime;
 			eval {
-				$mtime=rcs_getmtime("$config{srcdir}/$file");
+				$mtime=rcs_getmtime($file);
 			};
 			if ($@) {
 				print STDERR $@;
@@ -759,7 +759,10 @@ sub refresh () {
 	my ($new, $internal_new)=find_new_files($files);
 	my ($del, $internal_del)=find_del_files($pages);
 	my ($changed, $internal_changed)=find_changed($files);
-	run_hooks(needsbuild => sub { shift->($changed) });
+	run_hooks(needsbuild => sub {
+		my $ret=shift->($changed, [@$del, @$internal_del]);
+		$changed=$ret if ref $ret eq 'ARRAY';
+	});
 	my $oldlink_targets=calculate_old_links($changed, $del);
 
 	foreach my $file (@$changed) {
